@@ -9,6 +9,7 @@ __all__ = (
     "AbonentChargeDetail",
     "AbonentChargeDetailService",
     "AbonentChargeDetailInvoice",
+    "AbonentSaveIndication",
 )
 
 from datetime import date, datetime
@@ -28,7 +29,7 @@ from typing import (
 import attr
 
 from inter_rao_energosbyt.actions import ActionRequest, DataMapping
-from inter_rao_energosbyt.actions._bases import HierarchicalItemsBase
+from inter_rao_energosbyt.actions._bases import HierarchicalItemsBase, ResultCodeMappingBase
 from inter_rao_energosbyt.actions.sql import ACTION_SQL
 from inter_rao_energosbyt.converters import (
     conv_float_optional,
@@ -367,10 +368,10 @@ class AbonentEquipment(DataMapping):
     nm_service: str = attr.ib(converter=str)
     """Service name"""
 
-    nn_ind_receive_end: int = attr.ib(converter=int)
+    nn_ind_receive_end: Optional[int] = attr.ib(converter=conv_int_optional, default=None)
     """Indications receive end day (of the month)"""
 
-    nn_ind_receive_start: int = attr.ib(converter=int)
+    nn_ind_receive_start: Optional[int] = attr.ib(converter=conv_int_optional, default=None)
     """Indications receive start day (of the month)"""
 
     nn_pu: int = attr.ib(converter=int)
@@ -661,7 +662,7 @@ _AbonentChargeDetailBase.register_parse_type(0, AbonentChargeDetail)
 
 
 @attr.s(kw_only=True, frozen=True, slots=True)
-class AbonentSaveIndication(DataMapping):
+class AbonentSaveIndication(ResultCodeMappingBase):
     __slots__ = ()
 
     @classmethod
