@@ -1,3 +1,28 @@
+__all__ = (
+    "BaseEnergosbytAPI",
+    "Account",
+    "WithAccount",
+    "WithDatedRequests",
+    "AbstractAccountWithBalance",
+    "AbstractAccountWithIndications",
+    "AbstractAccountWithInvoices",
+    "AbstractAccountWithMeters",
+    "AbstractAccountWithPayments",
+    "AbstractAccountWithTariffHistory",
+    "AbstractCalculatableMeter",
+    "AbstractPayment",
+    "AbstractSubmittableMeter",
+    "AbstractBalance",
+    "AbstractInvoice",
+    "AbstractIndication",
+    "AbstractTariffHistoryEntry",
+    "AbstractMeterHistoryEntry",
+    "AbstractMeter",
+    "AbstractMeterZone",
+    "AbstractAccountWithMeterHistory",
+    "AccountID",
+    "SupportedAccountsType",
+)
 import asyncio
 import inspect
 import json
@@ -61,9 +86,8 @@ from inter_rao_energosbyt.util import (
     SupportsLessThan,
 )
 
-TransmittedIndicationsType = Mapping[str, Union[int, float]]
 MeterID = str
-AccountID = str
+AccountID = int
 
 
 #################################################################################
@@ -158,8 +182,8 @@ class Account(Generic[_TAPI]):
 
     @property
     @final
-    def id(self) -> str:
-        return str(self.data.id_service)
+    def id(self) -> AccountID:
+        return self.data.id_service
 
     @property
     @final
@@ -176,7 +200,8 @@ class Account(Generic[_TAPI]):
 
     @property
     def code(self) -> str:
-        return str(self.data.nn_ls) or str(self.id)
+        nn_ls = self.data.nn_ls
+        return (None if nn_ls is None else str(nn_ls)) or str(self.id)
 
     @property
     @final
@@ -355,14 +380,14 @@ class BaseEnergosbytAPI(ABC):
         password: str,
         user_agent: Optional[str] = None,
         max_request_attempts: int = 3,
-        max_simultaneous_requests: int = 20,
+        max_simultaneous_requests: int = 10,
     ):
         self.username: str = username
         self.password: str = password
         self.auth_session: Optional[Login] = None
         self.max_request_attempts: int = max_request_attempts
 
-        self._accounts: Optional[Dict[int, Account]] = None
+        self._accounts: Optional[Dict[AccountID, Account]] = None
 
         self._requests_counter: int = 0
         self.logger: logging.Logger = logging.getLogger(self.__class__.__qualname__)
@@ -384,6 +409,10 @@ class BaseEnergosbytAPI(ABC):
         if not self._session.closed:
             await self._session.close()
 
+    @property
+    def session(self) -> aiohttp.ClientSession:
+        return self._session
+
     #################################################################################
     # Abstract API guarding
     #################################################################################
@@ -469,6 +498,7 @@ class BaseEnergosbytAPI(ABC):
 
         for i in range(max(self.max_request_attempts, 1)):
             attempt = i + 1
+            status = -1
             try:
                 try:
 
@@ -481,10 +511,10 @@ class BaseEnergosbytAPI(ABC):
                         async with self._session.post(
                             request_url, data=post_data, raise_for_status=True
                         ) as response:
+                            status = response.status
                             response_text = await response.text()  # TODO: encoding required?
                             logger.debug(
-                                "[%d] <- (a%d) (%d) %s"
-                                % (counter, attempt, response.status, response_text)
+                                "[%d] <- (a%d) (%d) %s" % (counter, attempt, status, response_text)
                             )
 
                 except aiohttp.ClientError as e:
@@ -501,9 +531,9 @@ class BaseEnergosbytAPI(ABC):
 
                 return response_decoded
 
-            except EnergosbytException:
+            except EnergosbytException as e:
                 logger.error(
-                    "[%d] <- (a%d) (%d) (!!! ERROR !!!)" % (counter, attempt, response.status)
+                    "[%d] <- (a%d) (%d) (!!! ERROR !!!) %r" % (counter, attempt, status, e)
                 )
                 if attempt >= self.max_request_attempts:
                     raise
@@ -574,6 +604,11 @@ class BaseEnergosbytAPI(ABC):
         return self.auth_session is not None and self.auth_session.is_success
 
     async def async_authenticate(self) -> None:
+        # This is required to reset session cookie
+        self._session.cookie_jar.clear()
+        async with self._session.get(self.AUTH_URL) as response:
+            pass
+        
         response = (
             await Login.async_request(
                 self,
@@ -705,11 +740,13 @@ class BaseEnergosbytAPI(ABC):
 
         return response
 
-    async def async_get_questions(self, account_id: SupportsInt) -> Dict[int, str]:
-        response = await GetLSQuestions.async_request(
-            self,
-            id_service=int(account_id),
-        )
+    async def async_get_questions(
+        self, account_id: Union[AccountID, SupportsInt]
+    ) -> Dict[int, str]:
+        if not isinstance(account_id, AccountID):
+            account_id = int(account_id)
+
+        response = await GetLSQuestions.async_request(self, id_service=account_id)
 
         return {question_item.id_question: question_item.nm_question for question_item in response}
 
@@ -819,8 +856,8 @@ class BaseEnergosbytAPI(ABC):
         """Set account description.
 
         :param account_id: Account identifier
-        :param description: Text containing description. Anything evaluating to `False` is automatically
-                            assumed to be an empty description.
+        :param description: Text containing description. Anything evaluating to
+                            `False` is automatically assumed to be an empty description.
         :param update_accounts: Perform accounts update after successful request.
         """
         description = "" if description is None else str(description).strip()
@@ -846,7 +883,7 @@ class BaseEnergosbytAPI(ABC):
     #################################################################################
 
     @property
-    def accounts(self) -> Optional[Mapping[int, Account]]:
+    def accounts(self) -> Optional[Mapping[AccountID, Account]]:
         return None if self._accounts is None else MappingProxyType(self._accounts)
 
     def _create_account_from_data(self, account_data: "LSList") -> Account:
@@ -1126,6 +1163,9 @@ class AbstractInvoice(WithAccount["AbstractAccountWithInvoices"], SupportsLessTh
     def __str__(self) -> str:
         return f"{self.__class__.__name__}[{self.id}]({self.period.isoformat()}, {self.total})"
 
+    def __lt__(self, other: "AbstractInvoice") -> bool:
+        return self.period < other.period
+
     @property
     @abstractmethod
     def period(self) -> "date":
@@ -1168,6 +1208,10 @@ class AbstractInvoice(WithAccount["AbstractAccountWithInvoices"], SupportsLessTh
     def service(self) -> Optional[float]:
         return None
 
+    @property
+    def recalculations(self) -> Optional[float]:
+        return None
+
 
 _TInvoice = TypeVar("_TInvoice", bound=AbstractInvoice)
 
@@ -1265,7 +1309,7 @@ class _AbstractTransmittingMeterBase(AbstractMeter, ABC):
         if not ignore_values:
             zones = self.zones
             for zone_id, new_value in kwargs.items():
-                if new_value is None:
+                if new_value is None or zone_id not in zones:
                     continue
                 last_zone_indication = zones[zone_id].last_indication
                 if last_zone_indication is None:
@@ -1280,8 +1324,13 @@ class _AbstractTransmittingMeterBase(AbstractMeter, ABC):
             start, end = self.submission_period
             today = date.today()
 
-            if not (start >= today >= end):
-                raise EnergosbytException("out of period submisson")
+            if not (start <= today <= end):
+                raise EnergosbytException(
+                    f"out of period submisson "
+                    f"({today.isoformat()} "
+                    f"not in {start.isoformat()} :: "
+                    f"{end.isoformat()})"
+                )
 
         return kwargs
 
@@ -1295,7 +1344,7 @@ class AbstractSubmittableMeter(_AbstractTransmittingMeterBase, ABC):
     __slots__ = ()
 
     @abstractmethod
-    def _internal_async_submit_indications(self, **kwargs) -> Any:
+    async def _internal_async_submit_indications(self, **kwargs) -> Any:
         pass
 
     async def async_submit_indications(
@@ -1318,7 +1367,7 @@ class AbstractCalculatableMeter(_AbstractTransmittingMeterBase, ABC):
     __slots__ = ()
 
     @abstractmethod
-    def _internal_async_calculate_indications(self, **kwargs) -> SupportsFloat:
+    async def _internal_async_calculate_indications(self, **kwargs) -> SupportsFloat:
         pass
 
     async def async_calculate_indications(
