@@ -1,6 +1,6 @@
 from abc import ABC, abstractmethod
 from datetime import date, datetime
-from typing import Iterable, List, Mapping, Optional, TYPE_CHECKING, Tuple, final
+from typing import Any, Iterable, List, Mapping, Optional, TYPE_CHECKING, Tuple, Union, final
 
 import attr
 
@@ -8,6 +8,7 @@ from inter_rao_energosbyt.presets.containers import MeterContainer
 from inter_rao_energosbyt.converters import (
     conv_float_optional,
     conv_float_substitute,
+    conv_int_optional,
     conv_str_optional,
 )
 from inter_rao_energosbyt.exceptions import EnergosbytException
@@ -22,7 +23,9 @@ from inter_rao_energosbyt.interfaces import (
     AbstractAccountWithIndications,
     AbstractAccountWithInvoices,
     AbstractAccountWithPayments,
+    AbstractSubmittableMeter,
     Account,
+    WithAccount,
 )
 from inter_rao_energosbyt.actions.sql.abonent import (
     AbonentChargeDetail,
@@ -32,6 +35,7 @@ from inter_rao_energosbyt.actions.sql.abonent import (
     AbonentEquipment,
     AbonentIndications,
     AbonentPays,
+    AbonentSaveIndication,
 )
 from inter_rao_energosbyt.presets.adapters import AccountWithInvoicesToIndications
 from inter_rao_energosbyt.util import AnyDateArg, extrapolate_zone_id, process_start_end_arguments
@@ -173,9 +177,14 @@ class MeterZoneContainer(AbstractMeterZone):
 
 
 @attr.s(kw_only=True, frozen=True, slots=True, eq=True, order=False)
-class SmorodinaMeter(MeterContainer):
+class SmorodinaMeter(MeterContainer, WithAccount["AccountWithSmorodinaMeters"]):
     checkup: SmorodinaCheckupStatus = attr.ib()
     code: str = attr.ib()
+    zone_id: int = attr.ib()
+    meter_id: int = attr.ib()
+    billing_id: int = attr.ib()
+    _period_end_day: Optional[int] = attr.ib(converter=conv_int_optional, default=None)
+    _period_start_day: Optional[int] = attr.ib(converter=conv_int_optional, default=None)
 
     @classmethod
     def from_response(cls, account: "AccountWithSmorodinaMeters", data: "AbonentEquipment"):
@@ -183,29 +192,113 @@ class SmorodinaMeter(MeterContainer):
 
         today_indication: Optional[float] = None
         last_indications_date = data.dt_last_indication
+        last_indication = data.vl_last_indication
         if last_indications_date is not None:
             last_indications_date = datetime.fromisoformat(last_indications_date).date()
 
             if last_indications_date == date.today():
-                today_indication = data.vl_last_indication
+                today_indication = last_indication
 
         return cls(
             account=account,
-            id=str(data.id_pu),
+            id=str(data.id_counter) + "_" + str(data.id_counter_zn),
+            zone_id=data.id_counter_zn,
+            meter_id=data.id_counter,
+            billing_id=data.id_billing_counter,
+            period_start_day=data.nn_ind_receive_start,
+            period_end_day=data.nn_ind_receive_end,
             checkup=SmorodinaCheckupStatus(
                 date=checkup_date,
                 year=checkup_date.year,
             ),
             zones={
-                ("t" + str(data.id_counter_zn)): MeterZoneContainer(
+                ("t1"): MeterZoneContainer(
                     name=data.nm_service,
-                    last_indication=data.vl_last_indication,
+                    last_indication=last_indication,
                     today_indication=today_indication,
                 )
             },
+            last_indications_date=last_indications_date,
             code=data.nm_factory,
         )
 
+    @property
+    def submission_period(self) -> Tuple["date", "date"]:
+        today = date.today()
+
+        start_date = today.replace(day=self._period_start_day or 1)
+
+        period_end_day = self._period_end_day
+
+        if period_end_day is None:
+            end_date = today
+            day = 31
+            while True:
+                try:
+                    end_date = end_date.replace(day=day)
+                    break
+                except ValueError:
+                    day -= 1
+        else:
+            end_date = today.replace(day=period_end_day)
+
+        return (start_date, end_date)
+
+
+class AbstractSmorodinaSubmittableMeter(
+    SmorodinaMeter, AbstractSubmittableMeter, WithAccount["AccountWithSmorodinaMeters"], ABC
+):
+    async def async_submit_indications(
+        self,
+        *,
+        t1: Optional[Union[int, float]] = None,
+        ignore_periods: bool = False,
+        ignore_values: bool = False,
+        **kwargs,
+    ) -> Any:
+        return await super().async_submit_indications(
+            t1=t1,
+            ignore_periods=ignore_periods,
+            ignore_values=ignore_values,
+        )
+
+    @property
+    @abstractmethod
+    def smorodina_plugin_submit_indications(self) -> str:
+        pass
+
+    async def _internal_async_submit_indications(
+        self, t1: Optional[Union[int, float]] = None, **kwargs
+    ) -> Any:
+        _, provider = await self.account._internal_async_prepare_smorodina_preset_parameters()
+
+        if t1 is None:
+            t1 = self.zones["t1"].last_indication
+            if t1 is None:
+                t1 = 0.0
+
+        response = await AbonentSaveIndication.async_request(
+            self.account.api,
+            self.smorodina_plugin_submit_indications,
+            provider,
+            dt_indication=datetime.now().isoformat(),
+            id_counter=self.meter_id,
+            id_counter_zn=self.zone_id,
+            id_source=15418,  # predefined value from request ?
+            pr_skip_anomaly=1,
+            pr_skip_err=1,
+            vl_indication=t1,
+        )
+
+        if not response.is_success:
+            raise EnergosbytException(
+                "Could not submit indications",
+                response.kd_result,
+                response.nm_result,
+            )
+
+        return response.nm_result
+
 
 class AccountWithSmorodinaMeters(
     WithSmorodinaProxy, AbstractAccountWithMeters[SmorodinaMeter], ABC
@@ -409,11 +502,11 @@ class AccountWithSmorodinaInvoices(
     async def async_get_invoices(
         self, start: AnyDateArg = None, end: AnyDateArg = None
     ) -> List[SmorodinaInvoice]:
-        return await self.async_get_SmorodinaInvoice(start, end)
+        return await self.async_get_smorodina_invoices(start, end)
 
-    async def async_get_SmorodinaInvoice(
+    async def async_get_smorodina_invoices(
         self, start: AnyDateArg = None, end: AnyDateArg = None
-    ) -> Iterable[SmorodinaInvoice]:
+    ) -> List[SmorodinaInvoice]:
         start, end = process_start_end_arguments(start, end, self.timezone)
         proxy, provider = await self._internal_async_prepare_smorodina_preset_parameters()
         response = await AbonentChargeDetail.async_request(
@@ -484,7 +577,7 @@ class SmorodinaBalance(AbstractBalance):
         return self._data.sm_benefit
 
     @property
-    def recalc(self) -> Optional[float]:
+    def recalculations(self) -> Optional[float]:
         return self._data.sm_recalc
 
 
